%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%     DISCRETIONARY MONETARY POLICY IN THE CALVO MODEL    %
%         WILLEM VAN ZANDWEGHE AND ALEXANDER WOLMAN       % 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% CALIBRATION AND GRIDS
beta = 0.99;
alpha = 0.90;
epsilon = 10;
mu = 0;
nstar = 0.2;
markup = epsilon/(epsilon-1);
chi = 1/(markup*nstar^(1+mu));
%This program requires the CompEcon toolbox of Fackler and Miranda
nd = 10;
dmin = 1.0; dmax = 1.2;
fspace = fundefn('cheb',nd,dmin,dmax);
dgrid = funnode(fspace);
%
np0 = 500;
p0min = 0;
p0max = 2;
incrementp0 = (p0max-p0min)/(np0-1);
p0grid = (p0min:incrementp0:p0max)';
%
nm = 400;
mmin = 0.01;
mmax = 0.25;
incrementm = (mmax-mmin)/(nm-1);
mgrid = (mmin:incrementm:mmax)';
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STARTING VALUES PERIOD T+1: N_{T+1} = D_{T+1} = v_{T+1} = 0
maxfp = 5; %Retain up to maxfp fixed points
I = 2000;
HistH = zeros(I*nd,6+maxfp); %Retain history of I policy iterations
Histmfp = zeros(I,3+maxfp); %Retain all instances of multiple fixed points in final iteration
norm = 1;
iter = 0;
v0 = ones(nd,1);
Gamma0 = zeros(nd,1);
bigN0 = zeros(nd,1);
bigD0 = zeros(nd,1);
%Use Chebyshev polynomials to approximate the N, D, v function
cv = funfitxy(fspace,dgrid,v0);
cn = funfitxy(fspace,dgrid,bigN0);
cd = funfitxy(fspace,dgrid,bigD0);
%
eee_p0 = ones(np0,1);
eee_m = ones(nm,1);
eee_d = ones(nd,1);
dprime_p0 = dprime(eee_d*p0grid',dgrid,alpha,epsilon);
all_p0hat_m = zeros(nd,np0,nm);
mfp = [0 0 0]; %mfp(1)=1 if there are multiple fixed points (MFP) in any iteration, 
% mfp(2)=1 if there are MFP in the current iteration, 
% mfp(3)=1 if temporary equilibrium has MFP in any iteration
bigN0prime_p0 = zeros(nd,np0);
bigD0prime_p0 = zeros(nd,np0);
v0prime_m = zeros(nd,nm);
tstart = tic;
while norm > sqrt(eps)
    h = 0;
    iter = iter+1;
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STEP 2A: SOLVE FOR STATIONARY EQUILIBRIUM FOR ARBITRARY (Gamma,Delta)
    fp_m = -1*ones(nd,maxfp,nm);
    numfp_m = zeros(nd,nm);
    p0_m = zeros(nd,nm);
    mfp(2)=0;
    %Create matrices bigN0prime_p0, bigD0prime_p0 of dimension nd-times-np0
    for ip0 = 1:np0
        bigN0prime_p0(:,ip0) = funeval(cn,fspace,dprime_p0(:,ip0));
        bigD0prime_p0(:,ip0) = funeval(cd,fspace,dprime_p0(:,ip0));
    end
    %Compute temporary equilibrium and fixed points on grid of p0 and m
    for id = 1:nd
        c_p0_m = (eee_p0./inflation(p0grid,alpha,epsilon))*mgrid';
        dprime_p0_m = dprime_p0(id,:)'*eee_m';
        n_p0_m = dprime_p0_m .* c_p0_m;
        w_p0_m = chi * (n_p0_m.^mu) .* c_p0_m;
        num_m = (inflation(p0grid,alpha,epsilon)*eee_m').*(w_p0_m + alpha*beta*bigN0prime_p0(id,:)'*eee_m');
        den_m = (eee_p0 + alpha*beta*bigD0prime_p0(id,:)')*eee_m';
        p0hat_m = markup*num_m./den_m;
        %Linear interpolation y = y0+(x-x0)*(y1-y0)/(x1-x0) with x=y =>
        %x=y=(x1*y0-x0*y1)/(x1-x0-y1+y0)
        numstar_m = p0hat_m(1:np0-1,:).*(p0grid(2:np0)*eee_m')-p0hat_m(2:np0,:).*(p0grid(1:np0-1)*eee_m');
        denstar_m = (p0grid(2:np0)-p0grid(1:np0-1))*eee_m'-p0hat_m(2:np0,:)+p0hat_m(1:np0-1,:);
        p0star_m = numstar_m./denstar_m;
        %Select fixed points
        %If the deviation changes sign then there is a fixed point 
        dev_m = p0grid*eee_m' - p0hat_m; 
        a = find(dev_m==0);
        if isempty(a)==0
            error('zero in dev_m')
        end
        %Convert "changes sign" into "negative" 
        prod_m = dev_m(1:np0-1,:).*dev_m(2:np0,:);
        %Locate the negative elements
        [ir,ic] = find(prod_m<0);
        if (length(ic)==nm) & (ic==(1:nm)') %#ok<BDSCA,AND2>
            %There is a unique private-sector equilibrium for each element of m
            numfp_m(id,:) = eee_m';
            %Vectorize the indices that locate the fixed points
            ivec = sub2ind(size(p0star_m),ir,ic);
            %Select the fixed points 
            fp_m(id,1,:) = p0star_m(ivec);
            p0_m(id,:) = p0star_m(ivec);
        else %there are either no or multiple fixed points
            %Compute the number of fixed points for each element of m
            for jm = 1:nm   
                column =  [p0star_m(:,jm) prod_m(:,jm) ];
                index = find(column(:,2)<0.0);
                numfp = length(index);
                if numfp==0
                    error(['No fixed point for jm=' num2str(jm)]) 
                else
                    mfp(1) = 1;
                    mfp(2) = 1;
                    numfp_m(id,jm) = numfp;
                    fp_m(id,1:min(maxfp,numfp),jm) = column(index(1:min(maxfp,end)),1)';
                    %pick largest fixed point, as a simple way to continue
                    p0_m(id,jm) = fp_m(id,min(maxfp,numfp),jm);
                    %Alternatively, pick smallest fixed point
%                     p0_m(id,jm) = fp_m(id,1,jm);
                    if and(numfp>1,h<I)
                        h = h+1;
                        Histmfp(h,1:3+numfp) = [iter,id,jm,column(index(1:min(maxfp,end)),1)'];
                    end
                end
            end
        end
        all_p0hat_m(id,:,:) = p0hat_m;
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STEP 2B: SOLVE FOR POLICY FUNCTION AND VALUE FUNCTION BY PICKING Gamma FOR EACH Delta
    c_m = (eee_d*mgrid')./inflation(p0_m,alpha,epsilon);    
    dprime_m = dprime(p0_m,dgrid,alpha,epsilon);
    n_m = c_m.*dprime_m;
    u_m = log(c_m) - (chi/(1+mu))*n_m.^(1+mu);
    %Create matrix v0prime_m of dimension nd-times-nm
    for im = 1:nm
        v0prime_m(:,im) = funeval(cv,fspace,dprime_m(:,im));
    end
    v1_m = u_m + beta*v0prime_m;
    %Grid search
    [v1,index] = max(v1_m,[],2);
    Gamma1 = zeros(nd,1);
    p01 = zeros(nd,1);
    fp = zeros(nd,maxfp);
    for id = 1:nd
        Gamma1(id) = mgrid(index(id));
        p01(id) = p0_m(id,index(id));
        fp(id,:) = fp_m(id,:,index(id))';
        if max(numfp_m(id,index(id)))>1
            disp('Multiple fixed points, picked the p0 corresponding to the smallest m in argmax v')
            mfp(3) = 1;
        end
    end
    %Compute bigN0prime and bigD0prime
    dprimegrid = dprime(p01,dgrid,alpha,epsilon);
    bigN0prime = funeval(cn,fspace,dprimegrid);
    bigD0prime = funeval(cd,fspace,dprimegrid);
    %Compute bigN1 and bigD1
    c = Gamma1./inflation(p01,alpha,epsilon);
    n = dprimegrid.*c;
    w = chi*(n.^mu).*c;
    bigN1 = inflation(p01,alpha,epsilon).^epsilon.*(w + alpha*beta*bigN0prime);
    bigD1 = inflation(p01,alpha,epsilon).^(epsilon-1).*(eee_d + alpha*beta*bigD0prime);
%     p01 = markup*bigN1./bigD1;
    %sup norm
    norm_v = max(abs(v1-v0));
    norm_m = max(abs(Gamma1-Gamma0));
    norm_n = max(abs(bigN1-bigN0));
    norm_d = max(abs(bigD1-bigD0));
    norm = max(norm_v,norm_m);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % STEP 3: UPDATE STARTING VALUES
    v0 = v1;
    Gamma0 = Gamma1;
    %dampen the iteration (Judd, p. 558)
    bigN0 = 1*bigN1 + 0*bigN0;
    bigD0 = 1*bigD1 + 0*bigD0;
    cv = funfitxy(fspace,dgrid,v0);
    cn = funfitxy(fspace,dgrid,bigN0);
    cd = funfitxy(fspace,dgrid,bigD0);
    if iter<=I
        HistH((iter-1)*nd+1:iter*nd,:) = [iter*ones(nd,1),norm*ones(nd,1),v0,Gamma0,bigN0,bigD0,fp];
    end
    %Print output for this iteration
    disp(['iter = ' num2str(iter)])
    disp(['mfp = ' num2str(mfp) ',  norm_v = ' num2str(norm_v,'%0.2e') ',  norm_m = ' num2str(norm_m,'%0.2e') ',  norm_n,d = ' num2str(max(norm_n,norm_d),'%0.2e')])
%     subplot(3,2,1)
%     plot(dgrid,[dprimegrid,dgrid]')
%     title('State transition')
%     subplot(3,2,2)
%     plot(dgrid,p01)
%     title('p0 function')
%     subplot(3,2,3)
%     plot(dgrid,v1)
%     title('Value function')
%     subplot(3,2,4)
%     plot(dgrid,Gamma1)
%     title('Monetary policy function')
%     subplot(3,2,5)
%     plot(dgrid,[bigN1, bigN0prime ]')
%     title('N function')
%     subplot(3,2,6)
%     plot(dgrid,[bigD1, bigD0prime ]')
%     title('D function')
%    pause(0.01)
end
telapsed = toc(tstart);
k=100000;
max_abs_approx_error = residual_cheb(beta,chi,epsilon,alpha,mu,k,nd,dmin,dmax,cv,cn,cd,Gamma0,p01) %#ok<NOPTS>
[dss,piss] = steadystate_cheb(dmin,dmax,nd,p01,alpha,epsilon) %#ok<NOPTS>
display(dgrid)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% STEP 4: SAVE RESULTS
save sol_alpha09 iter norm nd nm dmin dmax cv cn cd dgrid mgrid p0grid...
    v0 Gamma0 bigN0 bigD0 p01 mfp v1_m u_m c_m dprime_m dprime_p0...
    all_p0hat_m HistH Histmfp 