% COMMITMENT TO DISCRETION
% Willem Van Zandweghe and Alexander L. Wolman

load sol_alpha05
nd = 12; dmin = 1.0; dmax = 1.061; 
fspace = fundefn('cheb',nd,dmin,dmax);

% Calibration
beta = 0.99;
alpha = 0.5;
eps = 10;
mu = 0;
nstar = 0.2;
mkp = eps/(eps-1);
chi = 1/(mkp*nstar^(1+mu));

% Iteration
T = 99;
delta = 1; 
Transition = zeros(T,10);
for t=1:T
    bigN = funeval(cn,fspace,delta);
    bigD = funeval(cd,fspace,delta);
    v = funeval(cv,fspace,delta);
    p0 = mkp*bigN/bigD;
    infln = ((1-alpha)*p0^(1-eps)+alpha)^(1/(1-eps));
    dprime = infln^eps*((1-alpha)*p0^(-eps)+alpha*delta); 
    m = interp1(dgrid,Gamma0,delta,'linear','extrap');
    c = m/infln;
    n = dprime*c;
    l = 1-n;
    u = log(c) - chi*n^(1+mu)/(1+mu);
    w = chi*c*n^mu;
    winv = 1/w;
    inflation = 100*(infln^4-1);
    Transition(t,:) = [t,inflation,winv,dprime,m,c,l,u,v,infln];
    delta = dprime;
end
% Add initial values associated with commitment steady state
infln0 = 1; w0 = 1/mkp; c0 = w0/chi; m0 = c0/infln0; n0 = c0; l0 = 1-n0;
u0 = log(c0)-(chi/(1+mu))*n0^(1+mu); v0 = u0/(1-beta);
inflation0 = 0;
Transition = [0 inflation0 mkp 1 m0 c0 l0 u0 v0 infln0; Transition];

filename = 'TableDC.xlsx';
sheet = 1;
xlRange = 'A1';
header ={'t','inflation','markup','rpd','m','c','l','u','v','infln'}; 
xlswrite(filename,header,sheet,xlRange);
xlRange = 'A2';
xlswrite(filename,Transition,sheet,xlRange);