%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Matlab figures for "Discretionary monetary policy in the Calvo model"   %
% Willem Van Zandweghe and Alexander L. Wolman                            %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%Figure 1: equilibrium
%%%%%%%%%%%%%%%%%%%%%%

load sol_alpha05
alpha=0.50; epsilon=10;
infl=inflation(p01,alpha,epsilon);
infl_annual = 100*(infl.^4-ones(nd,1));
viewdgrid=[1 1.005 1.01 1.015 1.02];
subplot(2,1,1)
[AX,H1,H2] = plotyy(dgrid,[dgrid dprime(p01,dgrid,alpha,epsilon)],dgrid,infl_annual);
title('A. State transition and inflation','FontSize',12)
xlabel('State variable (\Delta)','FontSize',12)
ylabel(AX(2),'    %','FontSize',12,'Rotation',0)
h_leg = legend([H1(2);H1(1);H2],'State transition (left) ','45-degree line (left) ',...
'Inflation (right) ','Location','northeast');
set(h_leg,'FontSize',11);
set(AX,'XLim',[1 1.02],'XTick',viewdgrid,'XTickLabel',viewdgrid,'FontSize',12)
set(AX(1),'YLim',[1 1.02],'YTick',viewdgrid,'YTickLabel',viewdgrid,'YColor',[0 0 0]);
set(AX(2),'YLim',[0 8],'YTick',[0 2 4 6 8],'YTickLabel',[0 2 4 6 8],...
    'YColor',[0 0 0]);
set(H1(1),'LineWidth',0.5,'Color',[0 0 0])
set(H1(2),'LineWidth',3,'Color',[0.3 0.3 1])
set(H2,'LineStyle','--','LineWidth',3,'Color',[0 0 0])
text(1.0022,1.0026,'\bullet','FontSize',18)
text(1.003,1.002,'Steady-state \Delta','FontSize',11)
text(1.003,1.001,'(left axis)','FontSize',11)
text(1.0022,1.0136,'\bullet','FontSize',18)
text(1.003,1.0153,'Steady-state inflation','FontSize',11)
text(1.003,1.0143,'rate (right axis)','FontSize',11)
subplot(2,1,2)
[AX,H1,H2] = plotyy(dgrid,Gamma0,dgrid,v0);
title('B. Policy instrument and welfare','FontSize',12)
xlabel('State variable (\Delta)','FontSize',12)
ylabel(AX(1),'\Gamma*(\Delta) ','FontSize',12,'Rotation',0)
ylabel(AX(2),' v*(\Delta)','FontSize',12,'Rotation',0)
h_leg = legend([H1;H2],'Policy instrument (left) ','Welfare (right) ','Location','northeast');
set(h_leg,'FontSize',11);
set(AX,'XLim',[1 1.02],'XTick',viewdgrid,'XTickLabel',viewdgrid,'FontSize',12)
set(AX(1),'YLim',[0.196 .205],'YTick',[0.196 0.199 0.202 0.205],'YTickLabel',...
    [0.196 0.199 0.202 0.205],'YColor',[0 0 0]);
set(AX(2),'YLim',[-251.19 -251.16],'YTick',[-251.19 -251.18 -251.17 -251.16],...
    'YTickLabel',[-251.19 -251.18 -251.17 -251.16],...
    'YColor',[0 0 0]);
set(H1,'LineWidth',3,'Color',[0.3 0.3 1])
set(H2,'LineStyle','--','LineWidth',3,'Color',[0 0 0])

set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperSize', 0.9*[8.5 11]);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperPosition', [0.01 0.01 0.85*8.5 0.89*11]);
set(gcf, 'renderer', 'painters');
print(gcf, '-dpdf', 'Fig1-equilibrium.pdf');

%Figure 2: best-response function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
load sol_alpha05
np0 = 400;
nm = 775;
% p0hat_m is computed for the 2nd node on the state grid (p0hat_m_d2),
% because it is equal to the steady state value of the state: 
% Delta = 1.0023 and Gamma(Delta) = 0.2022
p0hat_m_d2 = reshape(all_p0hat_m(2,:,:),np0,nm);
p0hat_m_d1 = reshape(all_p0hat_m(1,:,:),np0,nm);
p0hat_m_d3 = reshape(all_p0hat_m(3,:,:),np0,nm);
subplot(1,1,1)
h = plot(p0grid(2:end),[p0grid(2:end) p0hat_m_d2(2:end,621) ]);
xlabel('p_0','Fontsize',12)
viewp0grid = [0.8 0.9 1 1.1 1.2];
set(gca,'XLim',[viewp0grid(1) viewp0grid(end)],'XTick',viewp0grid,'XTickLabel',...
    viewp0grid,'YLim',[ 0.9 1.1 ],'YTick',[0.9 1 1.1],'YTickLabel',[0.9 1 1.1],...
    'FontSize',12)
set(h(1),'LineStyle',':','LineWidth',1,'Color',[0 0 0])
set(h(2),'LineWidth',2,'Color',[0.3 0.3 1])
h_leg = legend([h(2);h(1)],'Pricing best response','45-degree line','Location','northwest');
set(h_leg,'FontSize',11);
% 
set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperSize', [8.5 5.5]);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperPosition', [0.01 0.01 8.5 5.5]);
set(gcf, 'renderer', 'painters');
print(gcf, '-dpdf', 'Fig2-bestresponse.pdf');

%Figure 3: Transitions (see Data4Panel.xlsx)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
load Fig3Data.csv
y = Fig3Data(1:11,:);
x = (-2:8)';
X = [x,y];
t = X(:,1);
subplot(2,2,1)
h = plot(t,[X(:,8), X(:,9)]);
title('A. Nominal money growth rate','FontSize',12)
xlabel('Quarters','Fontsize',11)
ylabel('%','Fontsize',11,'Rotation',0)
set(gca,'XLim',[-2 8],'XTick',t,'XTickLabel',t,'YLim',[-6 12],...
    'YTick',[-6 0 6 12],'YTickLabel',[-6 0 6 12],'FontSize',12)
set(h(1),'LineWidth',3,'Color',[0.3 0.3 1])
set(h(2),'LineStyle','--','LineWidth',3,'Color',[0 0 0])
subplot(2,2,2)
h = plot(t,[X(:,2), X(:,3)]);
title('B. Inflation','FontSize',12)
xlabel('Quarters','Fontsize',11)
ylabel('%','Fontsize',11,'Rotation',0)
set(gca,'XLim',[-2 8],'XTick',t,'XTickLabel',t,'YLim',[-0.05 6],...
    'YTick',[0 2 4 6],'YTickLabel',[0 2 4 6],'FontSize',12)
set(h(1),'LineWidth',3,'Color',[0.3 0.3 1])
set(h(2),'LineStyle','--','LineWidth',3,'Color',[0 0 0])
subplot(2,2,3)
h = plot(t,[X(:,6), X(:,7)]);
title('C. Markup','FontSize',10)
xlabel('Quarters','Fontsize',11)
set(gca,'XLim',[-2 8],'XTick',t,'XTickLabel',t,'YLim',[1.09 1.12],...
    'YTick',[1.09 1.1 1.11 1.12],'YTickLabel',[1.09 1.1 1.11 1.12],'FontSize',12)
set(h(1),'LineWidth',3,'Color',[0.3 0.3 1])
set(h(2),'LineStyle','--','LineWidth',3,'Color',[0 0 0])
subplot(2,2,4)
h = plot(t,[X(:,4), X(:,5)]);
title('D. Relative price distortion','FontSize',12)
xlabel('Quarters','Fontsize',11)
set(gca,'XLim',[-2 8],'XTick',t,'XTickLabel',t,'YLim',[1 1.003],...
    'YTick',[1 1.001 1.002 1.003],'YTickLabel',[1 1.001 1.002 1.003],'FontSize',12)
set(h(1),'LineWidth',3,'Color',[0.3 0.3 1])
set(h(2),'LineStyle','--','LineWidth',3,'Color',[0 0 0])
h_leg = legend([h(1);h(2)],'Discretion to commitment',...
    'Commitment to discretion','Location','north');
set(h_leg,'FontSize',10.5);

set(gcf, 'PaperUnits', 'inches');
set(gcf, 'PaperSize', 0.9*[8.5 11]);
set(gcf, 'PaperPositionMode', 'manual');
set(gcf, 'PaperPosition', [0.01 0.01 0.85*8.5 0.89*11]);
set(gcf, 'renderer', 'painters');
print(gcf, '-dpdf', 'Fig3-transitions.pdf');


